package isi.servlet;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Properties;
import java.util.Vector;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class Acquisti extends HttpServlet{

	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema"; 
	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	private static Logger log = Logger.getLogger("isi.shop");
	
	@Override public void init(){
		try{
			Properties log4j = new Properties();
        	InputStream in = getClass().getClassLoader().getResourceAsStream("/logger.log4j.properties");
        	log4j.load(in);
        	PropertyConfigurator.configure(log4j);
		}
		catch(IOException e){
			System.out.println("Logger non avviato " + e.getMessage());
		}
	}
	
	private static final long serialVersionUID = 1L;
	@Override public void doPost(HttpServletRequest req, HttpServletResponse res){
		doGet(req,res);
	}
	
	@Override public void doGet(HttpServletRequest req, HttpServletResponse res){
		
		log.debug("Ricevuta richiesta");
		
		//Imposto il contentType della risposta
		res.setContentType("plain/text");
		
		//Prendo il Writer per scrivere la risposta
		PrintWriter out;
		try{
			out = res.getWriter();
		}
		catch(IOException ioe){
			try{
				res.sendError(500,"Impossibile scrivere una risposta!");
			}
			catch(IOException e){
				return;
			}
			return;
		}

		Carrello carrello = getCarrello(req);
		
		try{
			updateCarrello(req, carrello);
			setCarrello(carrello,req);
			
		}
		catch(IOException e){
			try{ res.sendError(500, "Errore nei diritti di accesso all'XSD: " + e.getMessage()); }catch(IOException ee){return;}
		}
		catch(ParserConfigurationException e){
			try{ res.sendError(500, "Errore di configurazione del parser: " + e.getMessage()); }catch(IOException ee){return;}
		}
		catch(SAXException e){
			try{ res.sendError(400, "Errore nel parsing dell'ordine: " + e.getMessage()); }catch(IOException ee){return;}
		}
		catch(Exception e){
			log.error("Errore generico",e);
			try{ res.sendError(500, "Errore generico: " + e.getMessage()); }catch(IOException ee){return;}
		}
		finally{
			printCarrello(out,carrello);
		}
		
		
	}
	
	
	/**
	 * Recupero il carrello della spesa dalla sessione
	 * @param HttpServletRequest req
	 * @return Carrello carrello
	 * @throws Exception
	 */
	
	private Carrello getCarrello(HttpServletRequest req){
		//Recupero la lista di parametri in sessione o la instanzio se ancora non esiste
		HttpSession session = req.getSession();
		Carrello carrello = null;
		if(session.getAttribute("carrello") != null){
			log.debug("Recupero della sessione.");
			carrello = (Carrello) (session.getAttribute("carrello"));
		}
		else{ 
			log.debug("Creazione nuova sessione. Carrello vuoto");
			carrello = new Carrello();
		}
		return carrello;
	}
	
	
	/**
	 * Imposta il carrello nella sessione
	 * 
	 * @param Carrello carrello 
	 * @param HttpServletRequest req
	 */
	private void setCarrello(Carrello carrello,HttpServletRequest req){
		//Inserisco la lista dei parametri aggiornata in sessione.
		req.getSession().setAttribute("carrello", carrello);
	}
	
	
	/**
	 * Scrive il contenuto del carrello nella risposta
	 * 
	 * @param Carrello carrello 
	 * @param HttpServletRequest req
	 */
	private void printCarrello(PrintWriter out, Carrello carrello){
		//Inserisco la lista dei parametri aggiornata in sessione.
		out.write(carrello.print());
	}
	
	
	/**
	 * Aggiorna il carrello con l'ordine ricevuto verificando la disponibilita' del conto.
	 * 
	 * @param HttpServletRequest req
	 * @param Carrello carrello
	 * @throws Exception
	 */
	private void updateCarrello(HttpServletRequest req, Carrello carrello) throws FileNotFoundException,ParserConfigurationException,SAXException,IOException,Exception {
		
		Document doc = validazioneOrdine(req);
		
		// Identita Cliente
		String idCliente = "["+doc.getFirstChild().getAttributes().getNamedItem("client").getNodeValue()+"]";
		
		//Cerco gli articoli nell'ordine e li aggiungo al carrello.
		NodeList articoli = doc.getElementsByTagName("ns:articolo");
		Vector<Articolo> articoliDaComprare = new Vector<Articolo>();
		if(articoli!=null && articoli.getLength()>0){
			for(int i = 0; i < articoli.getLength(); i++){
				Element articoloElement = (Element) articoli.item(i);
				Articolo articolo = new Articolo(articoloElement.getFirstChild().getTextContent(), Integer.parseInt(articoloElement.getLastChild().getTextContent()));
				log.info(idCliente+" Richiesta acquisto " + articolo.nome + ": pezzi n. " + articolo.quantita);
				articoliDaComprare.add(articolo);
			}
		}
		
		acquistaArticolo(articoliDaComprare,idCliente,carrello);
		
	}
	
	/**
	 * Costruisce l'albero dom con l'XML dell'ordine ricevuto. Effettua anche la validazione
	 * 
	 * @param HttpServletRequest req
	 * @return Document dom
	 * @throws Exception
	 */
	private Document validazioneOrdine(HttpServletRequest req) throws FileNotFoundException,ParserConfigurationException,SAXException,IOException,Exception{
		//Costruisco il DOM dell'ordine
		Document doc = null;
		log.info("Validazione XML dell'ordine...");
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			File file = new File(getServletContext().getRealPath("WEB-INF/ordine.xsd"));
			factory.setIgnoringElementContentWhitespace(true);
			factory.setNamespaceAware(true);
			factory.setValidating(true);
			factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
			factory.setAttribute(JAXP_SCHEMA_SOURCE, file);
			
			DocumentBuilder parser = factory.newDocumentBuilder();
			parser.setErrorHandler(new parserErrorHandler());
			doc = parser.parse(req.getInputStream());
		}
		catch(FileNotFoundException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(ParserConfigurationException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(SAXException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(IOException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(Exception e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		return doc;
	}
	

	private boolean acquistaArticolo(Vector<Articolo> articoliDaComprare,String idCliente,Carrello carrello){		
		
		Connection connectionDB = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		PreparedStatement pstmtAcquisizione = null;
		ResultSet rsAcquisizione = null;
		try{
			
			
			// Carica la classe del driver JDBC
			Class.forName(CostantiDatabase.DRIVER_JDBC);
			
			
			// Ottengo connessione al database
			connectionDB = DriverManager.getConnection( 
					CostantiDatabase.CONNECTION_URL,
					CostantiDatabase.USERNAME_ACCESSO_DATABASE,
					CostantiDatabase.PASSWORD_ACCESSO_DATABASE );
			if(connectionDB==null){
				log.error("Connection is null");
			}
			
			
			// gestisco la transazione applicativamente
			connectionDB.setAutoCommit(false);
						
			
							
			// Visualizzo listino degli articoli disponibili
			Vector<Articolo> articoliDisponibili = readArticoliDisponibiliInMagazzino(idCliente, connectionDB);
			
			
			
			// Check disponibilita
			// Se gli articoli da comprare non sono stati specificati significa che il cliente desidera comprare tutti gli articoli disponibili
			float fatturaArticoliRichiesti = fatturaArticoliRichiesti(idCliente, articoliDaComprare, articoliDisponibili);
			if(fatturaArticoliRichiesti==-1)
				return false;
			
			
			
			// Prima di procedere con l'acquisto l'applicazione shop attende 10 secondi.
			// Questa attesa simula una possibile interazione che puo' avvenire con il cliente.
			// In questa attesa il cliente deve decidere se procedere con l'acquisto della fattura visualizzata
			// Simuliamo che dopo attendere 10 secondi il client voglia procedere con l'acquisto
			log.info(idCliente+" Simulazione interazione con il client per richiedere se desidera procedere con l'acquisto....");
			simulazioneCalcoliInterni();

				
			// Si Procede con l'acquisto degli articoli indicati in fattura
			acquisizioneArticoli(idCliente, connectionDB, articoliDaComprare, carrello);
			
			
			// Commit delle operazioni effettuate sul database	
			log.info(idCliente+" Commit, salvo modifiche al database");
			connectionDB.commit();


		}
		catch(Exception e){
			
			carrello.clear();
			
			log.error(idCliente+" Errore generale: "+e.getMessage(),e);
			//e.printStackTrace();
			
			// rollback
			try{
				log.info(idCliente+" Rollback, annullo modifiche al database");
				connectionDB.rollback();
			}catch(Exception eRollback){}
			
			return false;
		}
		finally{
			
			try{
				
				// NOTA: chiudere sempre la connessione al database e ogni Statement/ResultSet utilizzato
				if( rs!=null )
					rs.close();
				if( pstmt!=null )
					pstmt.close();
				if( rsAcquisizione!=null )
					rsAcquisizione.close();
				if( pstmtAcquisizione!=null )
					pstmtAcquisizione.close();
				if( (connectionDB!=null) && (!connectionDB.isClosed()) ){
					connectionDB.close();
				}
			
			}catch(Exception eClose){
				log.error(idCliente+" Rilascio risorse non riuscito: "+eClose.getMessage());
			}
		}
		
		return true;
	}
	

	private Vector<Articolo> readArticoliDisponibiliInMagazzino(String idCliente,Connection connectionDB) throws Exception{
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Vector<Articolo> articoli = new Vector<Articolo>();
		try{
		
			
			
			log.info(idCliente+" ------------- Listino (Articoli) -------------");
			String sqlQuery = "SELECT * FROM "+CostantiDatabase.LISTINO;
			pstmt = connectionDB.prepareStatement(sqlQuery);
			rs = pstmt.executeQuery();
			log.info(idCliente+" ["+CostantiDatabase.LISTINO_CODICE_ARTICOLO+" - "+CostantiDatabase.LISTINO_DESCRIZIONE+" - "+CostantiDatabase.LISTINO_DISPONIBILITA+" - "+CostantiDatabase.LISTINO_PREZZO+"]");
			while(rs.next()){
				String codiceArticolo = rs.getString(CostantiDatabase.LISTINO_CODICE_ARTICOLO);
				int quantitaDisponibile = rs.getInt(CostantiDatabase.LISTINO_DISPONIBILITA);
				float prezzo = rs.getFloat(CostantiDatabase.LISTINO_PREZZO);
				log.info(idCliente+" ["+codiceArticolo+" - "
						+rs.getString(CostantiDatabase.LISTINO_DESCRIZIONE)
						+" - Num. "+quantitaDisponibile
						+" - €. "+prezzo+" ]");
				if(quantitaDisponibile>0){
					Articolo articolo = new Articolo(codiceArticolo,quantitaDisponibile,prezzo);
					articoli.add(articolo);
				}
			}	
			rs.close();
			pstmt.close();
			log.info(idCliente+" --------------------- Fine Listino ----------------------------\n");
			
		}finally{
			try{
				if( rs!=null )
					rs.close();
				if( pstmt!=null )
					pstmt.close();		
			}catch(Exception eClose){
				log.error(idCliente+" Rilascio risorse non riuscito: "+eClose.getMessage());
			}
		}
		return articoli;
	}
	
	private float fatturaArticoliRichiesti(String idCliente,Vector<Articolo> articoliDaComprare,Vector<Articolo> articoliDisponibili){
		log.info(idCliente+" --------------------- Fatturazione richiesta ----------------------------");
		float costoTotale = 0;
		if(articoliDaComprare.size()==0){
			if(articoliDisponibili.size()==0){
				log.info(idCliente+" Non vi sono prodotti non disponibili nel magazzino");
				log.info(idCliente+" --------------------- Fine fatturazione richiesta ----------------------------");
				return -1;
			}
			
			
			for(int i=0; i<articoliDisponibili.size();i++){
				Articolo articolo = articoliDisponibili.get(i);
				log.info(idCliente+" Num."+articolo.quantita+" di "+articolo.nome+" per un tot di E."+(articolo.quantita*articolo.prezzo));
				costoTotale += (articolo.quantita*articolo.prezzo);
			}
		}
		else{
			// Richiesta articoli puntuali
			for(int i=0; i<articoliDaComprare.size();i++){
				Articolo articoloDaComprare = articoliDaComprare.get(i);
				float prezzoArticoloTrovato = 0;
				for(int j=0; j<articoliDisponibili.size();j++){
					Articolo articoloInMagazzino = articoliDisponibili.get(j);
					if(articoloInMagazzino.nome.equals(articoloDaComprare.nome)){
						prezzoArticoloTrovato = articoloInMagazzino.prezzo;
						if(articoloInMagazzino.quantita<articoloDaComprare.quantita){
							log.info(idCliente+" Prodotto ["+articoloDaComprare.nome+"] non disponibile nella quantita richiesta "+articoloDaComprare.quantita
									+" (attuale giacenza:"+articoloInMagazzino.quantita+")");
							log.info(idCliente+" --------------------- Fine fatturazione richiesta ----------------------------");
							return -1;
						}
					}
				}
				if(prezzoArticoloTrovato==0){
					log.error(idCliente+" Prodotto ["+articoloDaComprare.nome+"] non esistente in magazzino");
					log.info(idCliente+" --------------------- Fine fatturazione richiesta ----------------------------");
					return -1;
				}
				log.info(idCliente+" Num."+articoloDaComprare.quantita+" di "+articoloDaComprare.nome+" per un tot di E."+(articoloDaComprare.quantita*prezzoArticoloTrovato));
				costoTotale += (articoloDaComprare.quantita*prezzoArticoloTrovato);
			}
		}
		log.info(idCliente+" Totale fattura E."+costoTotale);
		log.info(idCliente+" --------------------- Fine fatturazione richiesta ----------------------------");
		return costoTotale;
	}
	
	private void simulazioneCalcoliInterni(){
		try{
			Thread.sleep(10*1000);
		}catch(Exception e){}
	}
	
	
	
	private boolean acquisizioneArticoli(String idCliente,Connection connectionDB,Vector<Articolo> articoliDaComprare,Carrello carrello) throws Exception{
		
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		PreparedStatement pstmtAcquisizione = null;
		try{
			
			log.info(idCliente+" ------------- Resoconto Acquisto -------------");
			String sqlQuery = "SELECT * FROM "+CostantiDatabase.LISTINO;
			if(articoliDaComprare.size()>0){
				sqlQuery +=" WHERE ";
				for(int i=0; i<articoliDaComprare.size(); i++){
					if(i>0)
						sqlQuery +=" OR ";
					sqlQuery +=CostantiDatabase.LISTINO_CODICE_ARTICOLO+"=?";
				}
			}
			pstmt = connectionDB.prepareStatement(sqlQuery);
			if(articoliDaComprare.size()>0){
				sqlQuery +=" WHERE ";
				for(int i=0; i<articoliDaComprare.size(); i++){
					pstmt.setString((i+1), articoliDaComprare.get(i).nome);
				}
			}
			rs = pstmt.executeQuery();
			float totaleAcquisto = 0;
			log.info(idCliente+" ["+CostantiDatabase.LISTINO_CODICE_ARTICOLO+" - "+CostantiDatabase.LISTINO_DESCRIZIONE+" - "+CostantiDatabase.LISTINO_DISPONIBILITA+" - "+CostantiDatabase.LISTINO_PREZZO+"]");
			while(rs.next()){
				String codiceArticolo = rs.getString(CostantiDatabase.LISTINO_CODICE_ARTICOLO);
				int quantitaDisponibile = rs.getInt(CostantiDatabase.LISTINO_DISPONIBILITA);
				float prezzo = rs.getFloat(CostantiDatabase.LISTINO_PREZZO);
				Articolo articolo = new Articolo(codiceArticolo,quantitaDisponibile,prezzo);	
				
				// Aggiornamento quantita
				String updateQuery = "UPDATE "+CostantiDatabase.LISTINO+" set "+CostantiDatabase.LISTINO_DISPONIBILITA+"=? WHERE "+CostantiDatabase.LISTINO_CODICE_ARTICOLO+"=?";
				pstmtAcquisizione = connectionDB.prepareStatement(updateQuery);
				if(articoliDaComprare.size()>0){
					for(int i=0; i<articoliDaComprare.size(); i++){
						if(articoliDaComprare.get(i).nome.equals(codiceArticolo)){
							pstmtAcquisizione.setInt(1, (quantitaDisponibile-articoliDaComprare.get(i).quantita));	
							log.info(idCliente+" Num."+articoliDaComprare.get(i).quantita+" di "+codiceArticolo+" per un tot di E."+(articoliDaComprare.get(i).quantita*prezzo));
							totaleAcquisto += (articoliDaComprare.get(i).quantita*prezzo);
							articolo.quantita=articoliDaComprare.get(i).quantita;
							carrello.add(articolo);
							break;
						}
					}
				}
				else{
					pstmtAcquisizione.setInt(1, 0); // compro tutti
					log.info(idCliente+" Num."+quantitaDisponibile+" di "+codiceArticolo+" per un tot di E."+(quantitaDisponibile*prezzo));
					totaleAcquisto += (quantitaDisponibile*prezzo);
					carrello.add(articolo);
				}
				pstmtAcquisizione.setString(2, codiceArticolo);
				int entriesModificate = pstmtAcquisizione.executeUpdate();
				if(entriesModificate<0){
					log.error(idCliente+" Aggiornamento disponibilita "+codiceArticolo+", non riuscita, articolo non presente!");
					log.info(idCliente+" --------------------- Fine Resoconto Acquisto ----------------------------\n");
					return false;
				}				
	
			}
			rs.close();
			pstmt.close();
			log.info(idCliente+" Totale fattura E."+totaleAcquisto);
			log.info(idCliente+" --------------------- Fine Resoconto Acquisto ----------------------------\n");
			
		}finally{
			try{
				if( rs!=null )
					rs.close();
				if( pstmt!=null )
					pstmt.close();		
				if( pstmtAcquisizione!=null )
					pstmtAcquisizione.close();
			}catch(Exception eClose){
				log.error(idCliente+" Rilascio risorse non riuscito: "+eClose.getMessage());
			}
		}
		return true;
	}
	
	
	public class parserErrorHandler implements ErrorHandler{

		public void error(SAXParseException exception) throws SAXException {
			throw exception;
			
		}

		public void fatalError(SAXParseException exception) throws SAXException {
			throw exception;
		}

		public void warning(SAXParseException exception) throws SAXException {
			throw exception;
		}
	
		
	}
	
}

