package isi.servlet;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Properties;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class Magazzino extends HttpServlet{

	static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema"; 
	static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	private static Logger log = Logger.getLogger("isi.shop");
	
	@Override public void init(){
		try{
			Properties log4j = new Properties();
        	InputStream in = getClass().getClassLoader().getResourceAsStream("/logger.log4j.properties");
        	log4j.load(in);
        	PropertyConfigurator.configure(log4j);
		}
		catch(IOException e){
			System.out.println("Logger non avviato " + e.getMessage());
		}
	}
	
	private static final long serialVersionUID = 1L;
	@Override public void doPost(HttpServletRequest req, HttpServletResponse res){
		doGet(req,res);
	}
	
	@Override public void doGet(HttpServletRequest req, HttpServletResponse res){
		
		log.debug("Ricevuta richiesta");
		
		//Imposto il contentType della risposta
		res.setContentType("plain/text");
		
		//Prendo il Writer per scrivere la risposta
		PrintWriter out;
		try{
			out = res.getWriter();
		}
		catch(IOException ioe){
			try{
				res.sendError(500,"Impossibile scrivere una risposta!");
			}
			catch(IOException e){
				return;
			}
			return;
		}

		try{

			if(modificaMagazzino(req))
				out.write("Modifica del magazzino riuscita");	
			else
				out.write("Modifica del magazzino NON riuscita");
		}
		catch(Exception e){
			out.write("Modifica del magazzino NON riuscita: " + e.getMessage());
			try{ res.sendError(400, "Errore nel parsing dell'ordine: " + e.getMessage()); }catch(IOException ee){return;}
		}
		
	}
	
	

	
	
	/**
	 * Aggiorna il carrello con l'ordine ricevuto verificando la disponibilita' del conto.
	 * 
	 * @param HttpServletRequest req
	 * @param Carrello carrello
	 * @throws Exception
	 */
	private boolean modificaMagazzino(HttpServletRequest req) throws FileNotFoundException,ParserConfigurationException,SAXException,IOException {
		
		Document doc = validazioneOrdine(req);
		
		// Identita Cliente
		NodeList fornitore = doc.getElementsByTagName("ns:distributore");
		String idFornitore = "["+fornitore.item(0).getFirstChild().getTextContent()+"]";
		
		//Cerco gli articoli nell'ordine e li aggiungo al carrello.
		NodeList articoli = doc.getElementsByTagName("ns:articolo");
		
		for(int i = 0; i < articoli.getLength(); i++){
			Element articoloElement = (Element) articoli.item(i);
			NodeList list = articoloElement.getChildNodes();
			String codiceArticolo = null;
			String descrizioneArticolo = null;
			int quantitaArticolo = -1;
			float prezzoArticolo = -1f;
			for(int j=0; j<list.getLength(); j++){
				if("ns:nome".equals(list.item(j).getNodeName())){
					codiceArticolo = list.item(j).getTextContent();
				}else if("ns:descrizione".equals(list.item(j).getNodeName())){
					descrizioneArticolo = list.item(j).getTextContent();
				}else if("ns:quantita".equals(list.item(j).getNodeName())){
					quantitaArticolo = Integer.parseInt(list.item(j).getTextContent());
				}else if("ns:prezzo".equals(list.item(j).getNodeName())){
					prezzoArticolo = Float.parseFloat(list.item(j).getTextContent());
				}
			}
			log.info(idFornitore+" modifica articolo " + codiceArticolo + " con pezzi n. " + quantitaArticolo+" prezzo:"+prezzoArticolo);
			
			if( modificaMagazzino(codiceArticolo,quantitaArticolo,descrizioneArticolo,prezzoArticolo,idFornitore) ==false ){
				return false;
			}
			else{
				log.info(idFornitore+" Modifiche effettuate");
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Costruisce l'albero dom con l'XML dell'ordine ricevuto. Effettua anche la validazione
	 * 
	 * @param HttpServletRequest req
	 * @return Document dom
	 * @throws Exception
	 */
	private Document validazioneOrdine(HttpServletRequest req) throws FileNotFoundException,ParserConfigurationException,SAXException,IOException{
		//Costruisco il DOM dell'ordine
		Document doc = null;
		log.info("Validazione XML dell'ordine...");
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			File file = new File(getServletContext().getRealPath("WEB-INF/magazzino.xsd"));
			factory.setIgnoringElementContentWhitespace(true);
			factory.setNamespaceAware(true);
			factory.setValidating(true);
			factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
			factory.setAttribute(JAXP_SCHEMA_SOURCE, file);
			
			DocumentBuilder parser = factory.newDocumentBuilder();
			
			doc = parser.parse(req.getInputStream());
		}
		catch(FileNotFoundException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(ParserConfigurationException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(SAXException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		catch(IOException e){
			log.error("ERRORE! Validazione ordine fallita: " + e.getMessage());
			throw e;
		}
		return doc;
	}
	
	
	/**
	 * Verifica se il saldo del conto basta per pagare la cifra passata.
	 * @param int count
	 * @return boolean creditoSufficiente
	 */
	public boolean checkSaldo(int count){
		log.info("Verifico disponibilita in banca CC n. 123456...");
		//Bank_Client bank = new Bank_Client("123456",log);
		//int saldo = bank.saldo();
		return true;
	}
	
	
	

	
	
	public boolean modificaMagazzino(String nome,int quantita,String descrizione, float prezzo,String idCliente){		
		
		Connection connectionDB = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		PreparedStatement pstmtAcquisizione = null;
		ResultSet rsAcquisizione = null;
		try{
			
			// Carica la classe del driver JDBC
			Class.forName(CostantiDatabase.DRIVER_JDBC);
			
			// Ottengo connessione al database
			connectionDB = DriverManager.getConnection( 
					CostantiDatabase.CONNECTION_URL,
					CostantiDatabase.USERNAME_ACCESSO_DATABASE,
					CostantiDatabase.PASSWORD_ACCESSO_DATABASE );
			if(connectionDB==null){
				log.error("Connection is null");
			}
			
			 // Ogni operazione viene immediatamente riportata sulla base di dati
		    connectionDB.setAutoCommit(true);
			
			
			// Effettuo ricerca
			String sqlQuery = "SELECT "+CostantiDatabase.LISTINO_DISPONIBILITA+" FROM "+CostantiDatabase.LISTINO +" WHERE "+CostantiDatabase.LISTINO_CODICE_ARTICOLO+"=?";
			pstmt = connectionDB.prepareStatement(sqlQuery);
			pstmt.setString(1, nome);
			rs = pstmt.executeQuery();
			boolean articoloPresenteInMagazzino = rs.next();
			rs.close();
			pstmt.close();
			
			if(articoloPresenteInMagazzino==false){
				// Insert
				String insertQuery = "INSERT INTO "+CostantiDatabase.LISTINO+" VALUES (?,?,?,?)";
				pstmtAcquisizione = connectionDB.prepareStatement(insertQuery);
				pstmtAcquisizione.setString(1, nome);
				pstmtAcquisizione.setString(2, descrizione);
				pstmtAcquisizione.setInt(3, quantita);
				pstmtAcquisizione.setFloat(4, prezzo);
				int entriesModificate = pstmtAcquisizione.executeUpdate();
				if(entriesModificate<0){
					log.error(idCliente+" Aggiornamento disponibilita "+nome+", non riuscita, articolo non inserito");
					return false;
				}				
				log.info(idCliente+" "+quantita+" articoli ["+nome+"] sono stati aggiunti in magazzino con successo.");
			}
			else{
				// Update
				String updateQuery = "UPDATE "+CostantiDatabase.LISTINO+" set "+CostantiDatabase.LISTINO_DISPONIBILITA+"="+CostantiDatabase.LISTINO_DISPONIBILITA+"+? WHERE "+CostantiDatabase.LISTINO_CODICE_ARTICOLO+"=?";
				pstmtAcquisizione = connectionDB.prepareStatement(updateQuery);
				pstmtAcquisizione.setInt(1, (quantita));
				pstmtAcquisizione.setString(2, nome);
				int entriesModificate = pstmtAcquisizione.executeUpdate();
				if(entriesModificate<0){
					log.error(idCliente+" Aggiornamento disponibilita "+nome+", non riuscita, articolo non presente!");
					return false;
				}				
				log.info(idCliente+" "+quantita+" articoli ["+nome+"] sono stati aggiunti in magazzino con successo.");
			}

		}
		catch(Exception e){
			log.error(idCliente+" Errore generale: "+e.getMessage(),e);
			return false;
		}
		finally{
			
			try{
				
				// NOTA: chiudere sempre la connessione al database e ogni Statement/ResultSet utilizzato
				if( rs!=null )
					rs.close();
				if( pstmt!=null )
					pstmt.close();
				if( rsAcquisizione!=null )
					rsAcquisizione.close();
				if( pstmtAcquisizione!=null )
					pstmtAcquisizione.close();
				if( (connectionDB!=null) && (!connectionDB.isClosed()) ){
					connectionDB.close();
				}
			
			}catch(Exception eClose){
				log.error(idCliente+" Rilascio risorse non riuscito: "+eClose.getMessage());
			}
		}
		
		return true;
	}
	

	
}


